@@ -105,6 +105,7 @@ enum WacomErrorCode {
 	WERROR_INVALID_DB,	/**< The passed DB is invalid */
 	WERROR_BAD_ACCESS,	/**< Invalid permissions to access the path */
 	WERROR_UNKNOWN_MODEL,	/**< Unsupported/unknown device */
+	WERROR_BUG_CALLER,	/**< A bug in the caller */
 };
 
 /**
@@ -129,6 +130,9 @@ typedef enum {
 
 /**
  * Classes of devices.
+ *
+ * @deprecated This enum should no longer be used. The classes are not
+ * fine-grained or reliable enough to be useful.
  */
 typedef enum {
 	WCLASS_UNKNOWN,		/**< Unknown/unsupported device class */
@@ -158,8 +162,19 @@ typedef enum {
 	WSTYLUS_STROKE,
 	WSTYLUS_PUCK,
 	WSTYLUS_3D,
+	WSTYLUS_MOBILE,
 } WacomStylusType;
 
+/**
+ * Type of eraser on a stylus
+ */
+typedef enum {
+	WACOM_ERASER_UNKNOWN,
+	WACOM_ERASER_NONE,      /**< No eraser is present on the stylus */
+	WACOM_ERASER_INVERT,	/**< Eraser is a separate tool on the opposite end of the stylus */
+	WACOM_ERASER_BUTTON,	/**< Eraser is a button alongside any other stylus buttons */
+} WacomEraserType;
+
 /**
  * Capabilities of the various tablet buttons
  */
@@ -317,7 +332,7 @@ WacomDevice* libwacom_new_from_name(const WacomDeviceDatabase *db, const char *n
  * @return A NULL terminated list of pointers to all the devices inside the
  * database.
  * The content of the list is owned by the database and should not be
- * modified of freed. Use free() to free the list.
+ * modified or freed. Use free() to free the list.
  */
 WacomDevice** libwacom_list_devices_from_database(const  WacomDeviceDatabase *db, WacomError *error);
 
@@ -351,7 +366,11 @@ int libwacom_compare(const WacomDevice *a, const WacomDevice *b, WacomCompareFla
 /**
  * @param device The tablet to query
  * @return The class of the device
+ *
+ * @deprecated This function should no longer be used. The classes are not
+ * fine-grained or reliable enough to be useful.
  */
+LIBWACOM_DEPRECATED
 WacomClass libwacom_get_class(const WacomDevice *device);
 
 /**
@@ -360,6 +379,12 @@ WacomClass libwacom_get_class(const WacomDevice *device);
  */
 const char* libwacom_get_name(const WacomDevice *device);
 
+/**
+ * @param device The tablet to query
+ * @return The vendor-specific model name (e.g. CTE-650 for a Bamboo Fun), or NULL if none is set
+ */
+const char* libwacom_get_model_name(const WacomDevice *device);
+
 /**
  * @param device The tablet to query
  * @return The full filename including path to the SVG layout of the device
@@ -370,6 +395,10 @@ const char* libwacom_get_layout_filename(const WacomDevice *device);
 /**
  * @param device The tablet to query
  * @return The numeric vendor ID for this device
+ *
+ * @bug The return value is a signed int but libwacom_match_get_vendor_id()
+ * returns an unsigned int. This may cause compiler warnings, but the
+ * effective range for vendor IDs is 16-bit only anyway.
  */
 int libwacom_get_vendor_id(const WacomDevice *device);
 
@@ -406,6 +435,10 @@ const WacomMatch* libwacom_get_paired_device(const WacomDevice *device);
 /**
  * @param device The tablet to query
  * @return The numeric product ID for this device
+ *
+ * @bug The return value is a signed int but libwacom_match_get_product_id()
+ * returns an unsigned int. This may cause compiler warning, but the
+ * effective range for product IDs is 16-bit only anyway.
  */
 int libwacom_get_product_id(const WacomDevice *device);
 
@@ -580,6 +613,13 @@ int         libwacom_stylus_get_id (const WacomStylus *stylus);
  */
 const char *libwacom_stylus_get_name (const WacomStylus *stylus);
 
+/**
+ * @param stylus The stylus to query
+ * @param num_paired_ids The length of the returned list
+ * @return The list of other IDs paired to this stylus
+ */
+const int *libwacom_stylus_get_paired_ids(const WacomStylus *stylus, int *num_paired_ids);
+
 /**
  * @param stylus The stylus to query
  * @return The number of buttons on the stylus
@@ -587,14 +627,31 @@ const char *libwacom_stylus_get_name (const WacomStylus *stylus);
 int         libwacom_stylus_get_num_buttons (const WacomStylus *stylus);
 
 /**
+ * Check if the given stylus is paired with a separate eraser.
+ *
+ * If this function returns @c true then the tool described by the given
+ * WacomStylus is paired with a separate eraser tool. The actual eraser
+ * tool may be located by iterating over the list of paired styli.
+ *
  * @param stylus The stylus to query
- * @return Whether the stylus has an eraser
+ * @return Whether the stylus is paired with an eraser
+ * @see libwacom_stylus_get_paired_ids
+ * @see libwacom_stylus_is_eraser
  */
 int         libwacom_stylus_has_eraser (const WacomStylus *stylus);
 
 /**
+ * Check if the given stylus may act like an eraser.
+ *
+ * If this function returns @c true then the tool described by the given
+ * WacomStylus may act like an eraser. Such a tool may be dedicated to
+ * sending just eraser events (and paired with a separate tool for "tip"
+ * events) or capable of sending both both tip and eraser events.
+ *
  * @param stylus The stylus to query
- * @return Whether the stylus is actually an eraser
+ * @return Whether the stylus can act as an eraser
+ * @see libwacom_stylus_get_eraser_type
+ * @see libwacom_stylus_has_eraser
  */
 int         libwacom_stylus_is_eraser (const WacomStylus *stylus);
 
@@ -622,6 +679,12 @@ WacomAxisTypeFlags libwacom_stylus_get_axes (const WacomStylus *stylus);
  */
 WacomStylusType libwacom_stylus_get_type (const WacomStylus *stylus);
 
+/**
+ * @param stylus The stylus to query
+ * @return The type of eraser on the stylus
+ */
+WacomEraserType libwacom_stylus_get_eraser_type (const WacomStylus *stylus);
+
 /**
  * Print the description of this stylus to the given file.
  *
